﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Steema.TeeChart.WPF.Styles;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Windows.Markup;
using System.Collections.Specialized;
using System.Collections;
using Steema.TeeChart.WPF;
using System.Diagnostics;
using System.Reflection;
using System.Globalization;
using Steema.TeeChart.WPF.Tools;

namespace XamlControls
{
    /// <summary>
    /// Interaktionslogik für Chart.xaml
    /// </summary>
    [DefaultProperty("Series"), ContentProperty("Series")]
    public partial class Chart : UserControl
    {
        public Chart()
        {
            InitializeComponent();

            //LicenseManager.CurrentContext.GetSavedLicenseKey(typeof(Steema.TeeChart.WPF.TChart), Assembly.GetExecutingAssembly());
            //LicenseManager.CurrentContext.GetSavedLicenseKey(typeof(Steema.TeeChart.WPF.Chart), Assembly.GetExecutingAssembly());

            tchart.Aspect.View3D = false;
            tchart.Axes.Left.MaximumOffset = 5;
            tchart.Axes.Right.MaximumOffset = 5;
            //tchart.Axes.Left.Labels.ValueFormat = "###,###.00";            
            //tchart.Axes.Right.Labels.ValueFormat = "###,###.00";            
            tchart.Legend.Alignment = LegendAlignments.Top;
            tchart.Legend.LegendStyle = LegendStyles.Series;
            tchart.BeforeDraw += new PaintChartEventHandler(tchart_BeforeDraw);
            tchart.AfterDraw += new PaintChartEventHandler(tchart_AfterDraw);

            //tchart.Width = Width;
            //tchart.Height = Height;
            
            Series.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(Series_CollectionChanged);
        }

        void tchart_BeforeDraw(object sender, Steema.TeeChart.WPF.Drawing.Graphics3D g)
        {
            if (tchart != null && tchart.Chart != null)
            {
                MarksTip _mt = null;

                if (tchart.Chart.Tools != null)
                {
                    lock (tchart.Series)
                    {
                        for (int _i = 0; _i < tchart.Series.Count; _i++)
                        {
                            if (tchart.Tools.Count > _i)
                            {
                                _mt = tchart.Tools[_i] as MarksTip;
                            }

                            if (_mt != null && tchart.Series[_i].Count > 0)
                            {
                                _mt.Series = tchart.Series[_i];
                            }
                            else
                            {
                                _mt.Series = null;
                            }

                        }
                    }
                }
            }
        }

        void tchart_AfterDraw(object sender, Steema.TeeChart.WPF.Drawing.Graphics3D g)
        {

        }

        ~Chart()
        {
            if (tchart != null && tchart.Chart != null)
            {
                tchart.Chart.Tools.Clear();
            }
        }

        void Series_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            try
            {
                if (e.Action == NotifyCollectionChangedAction.Add)
                {
                    IList _items = e.NewItems;

                    if (_items != null)
                    {
                        lock (_items)
                        {
                            foreach (object _item in _items)
                            {
                                if (_item != null)
                                {
                                    if (_item is Series)
                                    {
                                        Series _newseries = _item as Series;

                                        if (_newseries != null && _newseries.SeriesObject != null)
                                        {
                                            //_newseries.Resources = Resources;
                                            _newseries.DataContext = DataContext;

                                            lock (tchart.Series)
                                            {
                                                if (tchart.Series.IndexOf(_newseries.SeriesObject) == -1)
                                                {
                                                    tchart.Series.Add(_newseries.SeriesObject);

                                                    MarksTip _mt = new MarksTip(tchart.Chart);
                                                    _mt.MouseAction = MarksTipMouseAction.Click;
                                                    _mt.Series = _newseries.SeriesObject;
                                                    _mt.Style = MarksStyles.XY;
                                                    _mt.Active = true;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            catch (Exception _e)
            {
                Trace.WriteLine(_e.Message + " " + _e.StackTrace);
            }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public ObservableCollection<Series> Series
        {
            get
            {
                return (ObservableCollection<Series>)this.GetValue(SeriesProperty);
            }
            set
            {
                this.SetValue(SeriesProperty, value);
            }
        }

        private static readonly DependencyProperty SeriesProperty = DependencyProperty.Register
        (
          "Series",
          typeof(ObservableCollection<Series>),
          typeof(Chart),
          new FrameworkPropertyMetadata
          (
              new ObservableCollection<Series>(),
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault | 
              FrameworkPropertyMetadataOptions.AffectsRender |
              FrameworkPropertyMetadataOptions.AffectsArrange,
              new PropertyChangedCallback(OnSeriesChanged)
          )
        );

        public bool Clear
        {
            get { return (bool)this.GetValue(ClearProperty); }
            set { this.SetValue(ClearProperty, value); }
        }

        public static readonly DependencyProperty ClearProperty = DependencyProperty.Register
        (
          "Clear",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              false,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onClearChanged)
              )
        );

        public int TimeSpanMinutes
        {
            get { return (int)this.GetValue(TimeSpanMinutesProperty); }
            set { this.SetValue(TimeSpanMinutesProperty, value); }
        }

        public static readonly DependencyProperty TimeSpanMinutesProperty = DependencyProperty.Register
        (
          "TimeSpanMinutes",
          typeof(int),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              5,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onTimeSpanMinutesChanged)
              )
        );

        public bool View3D
        {
            get { return (bool)this.GetValue(View3DProperty); }
            set { this.SetValue(View3DProperty, value); }
        }

        public static readonly DependencyProperty View3DProperty = DependencyProperty.Register
        (
          "View3D",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              false,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onView3DChanged)
              )
        );

        public string Title
        {
            get { return (string)this.GetValue(TitleProperty); }
            set { this.SetValue(TitleProperty, value); }
        }

        public static readonly DependencyProperty TitleProperty = DependencyProperty.Register
        (
          "Title",
          typeof(string),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              null,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onTitleChanged)
              )
        );

        public bool YLeftAuto
        {
            get { return (bool)this.GetValue(YLeftAutoProperty); }
            set { this.SetValue(YLeftAutoProperty, value); }
        }

        public static readonly DependencyProperty YLeftAutoProperty = DependencyProperty.Register
        (
          "YLeftAuto",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              true,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYLeftAutoChanged)
              )
        );

        public bool YRightAuto
        {
            get { return (bool)this.GetValue(YRightAutoProperty); }
            set { this.SetValue(YRightAutoProperty, value); }
        }

        public static readonly DependencyProperty YRightAutoProperty = DependencyProperty.Register
        (
          "YRightAuto",
          typeof(bool),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              true,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYRightAutoChanged)
              )
        );

        public double YLeftMin
        {
            get { return (double)this.GetValue(YLeftMinProperty); }
            set { this.SetValue(YLeftMinProperty, value); }
        }

        public static readonly DependencyProperty YLeftMinProperty = DependencyProperty.Register
        (
          "YLeftMin",
          typeof(double),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              0.0,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYLeftMinChanged)
              )
        );

        public double YLeftMax
        {
            get { return (double)this.GetValue(YLeftMaxProperty); }
            set { this.SetValue(YLeftMaxProperty, value); }
        }

        public static readonly DependencyProperty YLeftMaxProperty = DependencyProperty.Register
        (
          "YLeftMax",
          typeof(double),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              0.0,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYLeftMaxChanged)
              )
        );

        public double YRightMin
        {
            get { return (double)this.GetValue(YRightMinProperty); }
            set { this.SetValue(YRightMinProperty, value); }
        }

        public static readonly DependencyProperty YRightMinProperty = DependencyProperty.Register
        (
          "YRightMin",
          typeof(double),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              0.0,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYRightMinChanged)
              )
        );

        public double YRightMax
        {
            get { return (double)this.GetValue(YRightMaxProperty); }
            set { this.SetValue(YRightMaxProperty, value); }
        }

        public static readonly DependencyProperty YRightMaxProperty = DependencyProperty.Register
        (
          "YRightMax",
          typeof(double),
          typeof(Chart),
          new FrameworkPropertyMetadata
              (
              0.0,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onYRightMaxChanged)
              )
        );

        public DateTime? XMinDate
        {
            get { return (DateTime?)this.GetValue(XMinDateProperty); }
            set { this.SetValue(XMinDateProperty, value); }
        }

        public static readonly DependencyProperty XMinDateProperty = DependencyProperty.Register
         (
           "XMinDate",
           typeof(DateTime?),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               null,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onXMinDateChanged)
               )
         );

        public DateTime? XMaxDate
        {
            get { return (DateTime?)this.GetValue(XMaxDateProperty); }
            set { this.SetValue(XMaxDateProperty, value); }
        }

        public static readonly DependencyProperty XMaxDateProperty = DependencyProperty.Register
         (
           "XMaxDate",
           typeof(DateTime?),
           typeof(Chart),
           new FrameworkPropertyMetadata
               (
               null,
               FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
               new PropertyChangedCallback(onXMaxDateChanged)
               )
         );

        static void onXMinDateChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    if (_data is DateTime)
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Bottom.Minimum = ((DateTime)_data).ToOADate();
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void onClearChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                if (e.NewValue is bool)
                {
                    if ((bool)e.NewValue)
                    {
                        _sender_ctrl.tchart.Series.Clear();
                    }
                }
            }
        }

        static void onXMaxDateChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    if (_data is DateTime)
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Bottom.Maximum = ((DateTime)_data).ToOADate();
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void OnSeriesChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
        }

        static void onYRightMaxChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    double _val;

                    if (double.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Right.Maximum = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void onYRightMinChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    double _val;

                    if (double.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Right.Minimum = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void onYLeftMaxChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    double _val;

                    if (double.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Left.Maximum = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void onYLeftMinChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    double _val;

                    if (double.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Left.Minimum = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void onYLeftAutoChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Left.Automatic = _val;

                            // wenn Achsen manuell skaliert werden, dann die eingegebenen Grenzen wieder setzen
                            if (!_val)
                            {
                                _sender_ctrl.tchart.Axes.Left.Minimum = _sender_ctrl.YLeftMin;
                                _sender_ctrl.tchart.Axes.Left.Maximum = _sender_ctrl.YLeftMax;
                            }

                            // Chart neu zeichnen - wird nicht veranlasst, bei Setzen des Automatik-Modus !!!
                            _sender_ctrl.tchart.DoInvalidate();
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void onYRightAutoChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Axes.Right.Automatic = _val;

                            // wenn Achsen manuell skaliert werden, dann die eingegebenen Grenzen wieder setzen
                            if (!_val)
                            {
                                _sender_ctrl.tchart.Axes.Right.Minimum = _sender_ctrl.YRightMin;
                                _sender_ctrl.tchart.Axes.Right.Maximum = _sender_ctrl.YRightMax;
                            }

                            // Chart neu zeichnen - wird nicht veranlasst, bei Setzen des Automatik-Modus !!!
                            _sender_ctrl.tchart.DoInvalidate();
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void onTimeSpanMinutesChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    if (_data is int)
                    {
                        int _val = (int)_data;

                        if (_val >= 0)
                        {
                            switch (_val)
                            {
                                case 1:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.FifteenSeconds);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm:ss";

                                    break;

                                case 5:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneMinute);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm:ss";

                                    break;

                                case 15:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneMinute);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";

                                    break;

                                case 30:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.FiveMinutes);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";

                                    break;

                                case 60:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.FifteenMinutes);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";

                                    break;

                                case 120:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.ThirtyMinutes);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";

                                    break;

                                case 180:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.ThirtyMinutes);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";

                                    break;

                                case 360:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneHour);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";

                                    break;

                                case 720:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneHour);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";

                                    break;

                                case 1440:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneHour);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm";

                                    break;

                                case 2880:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.TwelveHours);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "dd.MM.yyyy HH:mm";

                                    break;

                                case 10080:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneDay);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "dd.MM.yyyy HH:mm";

                                    break;

                                default:

                                    _sender_ctrl.tchart.Axes.Bottom.Increment = Utils.GetDateTimeStep(DateTimeSteps.OneMinute);
                                    _sender_ctrl.tchart.Axes.Bottom.Labels.DateTimeFormat = "HH:mm:ss";

                                    break;
                            }

                            //DateTime _now = DateTime.Now;
                            //DateTime _past = _now.Subtract(TimeSpan.FromMinutes(_val));

                            //try
                            //{
                            //    _sender_ctrl.tchart.Axes.Bottom.SetMinMax(_past, _now);
                            //}
                            //catch (Exception _e)
                            //{
                            //    Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                            //}
                        }
                    }
                }
            }
        }

        static void onView3DChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.tchart.Aspect.View3D = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void onTitleChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Chart _sender_ctrl = sender as Chart;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    string _txt = _data.ToString();

                    try
                    {
                        if (!String.IsNullOrEmpty(_txt))
                        {
                            _sender_ctrl.tchart.Header.Visible = true;
                            _sender_ctrl.tchart.Header.Text = _txt;
                        }
                        else
                        {
                            _sender_ctrl.tchart.Header.Visible = false;
                        }
                    }
                    catch (Exception _e)
                    {
                        Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                    }
                }
            }
        }
    }
}
