﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Diagnostics;

namespace XamlControls
{
    public class Series : UserControl
    {
        static Series()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(Series), new FrameworkPropertyMetadata(typeof(Series)));

            VisibilityProperty.OverrideMetadata(
                typeof(Series),
                new FrameworkPropertyMetadata
                  (
                  Visibility.Visible,
                  FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
                  new PropertyChangedCallback(onVisibilityChanged)
                  )
                  );
        }

        public virtual Steema.TeeChart.WPF.Styles.Series SeriesObject { get; set; }

        public string Title
        {
            get { return (string)this.GetValue(TitleProperty); }
            set { this.SetValue(TitleProperty, value); }
        }

        public static readonly DependencyProperty TitleProperty = DependencyProperty.Register
        (
          "Title",
          typeof(string),
          typeof(Series),
          new FrameworkPropertyMetadata
              (
              "new series",
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onTitleChanged)
              )
        );

        public Color Color
        {
            get { return (Color)this.GetValue(ColorProperty); }
            set { this.SetValue(ColorProperty, value); }
        }

        public static readonly DependencyProperty ColorProperty = DependencyProperty.Register
        (
          "Color",
          typeof(Color),
          typeof(Series),
          new FrameworkPropertyMetadata
              (
              Colors.LimeGreen,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onColorChanged)
              )
        );

        public bool IsDateTime
        {
            get { return (bool)this.GetValue(IsDateTimeProperty); }
            set { this.SetValue(IsDateTimeProperty, value); }
        }

        public static readonly DependencyProperty IsDateTimeProperty = DependencyProperty.Register
        (
          "IsDateTime",
          typeof(bool),
          typeof(Series),
          new FrameworkPropertyMetadata
              (
              false,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onIsDateTimeChanged)
              )
        );

        public bool UseRightAxis
        {
            get { return (bool)this.GetValue(UseRightAxisProperty); }
            set { this.SetValue(UseRightAxisProperty, value); }
        }

        public static readonly DependencyProperty UseRightAxisProperty = DependencyProperty.Register
        (
          "UseRightAxis",
          typeof(bool),
          typeof(Series),
          new FrameworkPropertyMetadata
              (
              false,
              FrameworkPropertyMetadataOptions.BindsTwoWayByDefault,
              new PropertyChangedCallback(onUseRightAxisChanged)
              )
        );

        static void onUseRightAxisChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Series _sender_ctrl = sender as Series;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            if (_val)
                            {
                                _sender_ctrl.SeriesObject.VertAxis = Steema.TeeChart.WPF.Styles.VerticalAxis.Right;
                            }
                            else
                            {
                                _sender_ctrl.SeriesObject.VertAxis = Steema.TeeChart.WPF.Styles.VerticalAxis.Left;
                            }
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void onIsDateTimeChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Series _sender_ctrl = sender as Series;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    bool _val;

                    if (bool.TryParse(_data.ToString(), out _val))
                    {
                        try
                        {
                            _sender_ctrl.SeriesObject.XValues.DateTime = _val;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void onColorChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Series _sender_ctrl = sender as Series;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    if (_data is Color)
                    {
                        try
                        {
                            _sender_ctrl.SeriesObject.Color = (Color)_data;
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                }
            }
        }

        static void onTitleChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Series _sender_ctrl = sender as Series;

            if (_sender_ctrl != null)
            {
                object _data = e.NewValue;

                if (_data != null)
                {
                    try
                    {
                        _sender_ctrl.SeriesObject.Title = _data.ToString();
                    }
                    catch (Exception _e)
                    {
                        Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                    }
                }
            }
        }

        static void onVisibilityChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            Series _sender_ctrl = sender as Series;

            if (_sender_ctrl != null)
            {
                int _intval;
                object _data = e.NewValue;

                if (_data != null)
                {
                    if (_data is Visibility)
                    {
                        try
                        {
                            switch ((Visibility)_data)
                            {
                                case Visibility.Visible:

                                    _sender_ctrl.SeriesObject.Visible = true;

                                    break;

                                case Visibility.Hidden:
                                case Visibility.Collapsed:

                                    _sender_ctrl.SeriesObject.Visible = false;

                                    break;
                            }
                        }
                        catch (Exception _e)
                        {
                            Trace.WriteLine(_e.Message + " " + _e.StackTrace);
                        }
                    }
                    else if (int.TryParse(_data.ToString(), out _intval))
                    {
                        _sender_ctrl.SeriesObject.Visible = _intval > 0;
                    }
                }
            }
        }
    }
}
